﻿/*----------------------------------------------------------------
//  <copyright file="DemoController.cs" company="EaCloud项目团队">
//      Copyright (c) 2020-2021 SeonSoft. All rights reserved.
//  </copyright>
//  <site>http://www.seonsoft.biz</site>
//  <last-editor>Seon.Hu</last-editor>
//  <last-date>2021-04-17 19:06</last-date>
//----------------------------------------------------------------*/

using EaCloud;
using EaCloud.AspNetCore.UI;
using EaCloud.Authorization;
using EaCloud.Authorization.Modules;
using EaCloud.Net;
using EaCloud.Pack.Identity.Core.Dtos;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using System;
using System.ComponentModel;
using System.Security;
using System.Threading.Tasks;

namespace EaCloud.Template.Api.Demo.Controllers
{
    #region "网站-Demo"
    /// <summary>
    /// 网站-Demo
    /// </summary>
    [AllowAnonymous]
    [ClassFilter]
    [Description("网站-Demo")]
    [ApiExplorerSettings(GroupName = "buss")]
    [ModuleInfo(Code = "Demo", Name = "样例", Order = 99)]
    public class DemoController : DemoControllerBase
    {

        #region "Action方法"
        #region "获取当前登录用户名"
        /// <summary>
        /// 获取当前登录用户名
        /// </summary>
        [HttpGet]
        [LoggedIn]
        [MethodFilter]
        [Description("获取当前登录用户名")]
        public async Task<AjaxResult> GetLoginedUserName()
        {
            if (!User.Identity.IsAuthenticated)
            {
                return new AjaxResult("User Is Not Authenticated", AjaxResultType.Forbidden);
            }
            return await Task.FromResult(new AjaxResult("OK", AjaxResultType.Success, User.Identity.GetUserName()));
        }
        #endregion
        #region "获取当前登录组织名"
        /// <summary>
        /// 获取当前登录组织名
        /// </summary>
        [HttpGet]
        [LoggedIn]
        [MethodFilter]
        [Description("获取当前登录组织名")]
        public async Task<AjaxResult> GetLoginedOrgName()
        {
            if (!User.Identity.IsAuthenticated)
            {
                return new AjaxResult("User Is Not Authenticated", AjaxResultType.Forbidden);
            }
            return await Task.FromResult(new AjaxResult("OK", AjaxResultType.Success, User.Identity.GetCurrentOrganizationName()));
        }
        #endregion
        #region "获取当前登录父级组织"
        /// <summary>
        /// 获取当前登录父级组织
        /// </summary>
        [HttpGet]
        [LoggedIn]
        [MethodFilter]
        [Description("获取当前登录父级组织")]
        public async Task<AjaxResult> GetParentOrganizations()
        {
            if (!User.Identity.IsAuthenticated)
            {
                return new AjaxResult("User Is Not Authenticated", AjaxResultType.Forbidden);
            }
            return await Task.FromResult(new AjaxResult("OK", AjaxResultType.Success, ParentOrganizations));
        }
        #endregion

        #region "获取HttpClient"
        /// <summary>
        /// 获取HttpClient
        /// </summary>
        [HttpGet]
        [RoleLimit]
        [MethodFilter]
        [Description("获取HttpClient")]
        public async Task<AjaxResult> GetHttpClient()
        {
            var httpClient = CreateHttpClient();
            return await Task.FromResult(new AjaxResult("OK", AjaxResultType.Success, nameof(httpClient)));
        }
        #endregion

        #region "发送邮件"
        /// <summary>
        /// 发送邮件
        /// </summary>
        /// <param name="dto">邮箱信息</param>
        /// <returns>JSON操作结果</returns>
        [HttpPost]
        [AllowAnonymous]
        [MethodFilter]
        [DependOnFunction("CheckEmailExists")]
        [Description("发送激活注册邮件")]
        public async Task<AjaxResult> SendConfirmMail(SendMailDto dto)
        {
            if (!ModelState.IsValid)
            {
                return new AjaxResult("提交信息验证失败", AjaxResultType.Error);
            }
            IEmailSender sender = ServiceProvider.GetService<IEmailSender>();
            var subjectPrefix = ServiceProvider.GetEaCloudOptions()?.MailSender?.SubjectPrefix.ReplaceNull();
            string url = $"{Request.Scheme}://{Request.Host}";
            string body =
             $"亲爱的用户 <strong>{dto.Email}</strong>[{dto.Email}]，您好！<br>"
             + $"欢迎使用，网站链接请 <a href=\"{url}\" target=\"_blank\"><strong>点击这里</strong></a><br>"
             + $"如果上面的链接无法点击，您可以复制以下地址，并粘贴到浏览器的地址栏中打开。<br>"
             + $"{url}<br>"
             + $"祝您使用愉快！";
            await sender.SendEmailAsync(dto.Email, $"{subjectPrefix}Demo测试邮件", body);
            return new AjaxResult("激活Email邮件发送成功");
        }
        #endregion

        #endregion

    }
    #endregion

    public class ClassFilter : ActionFilterAttribute, IExceptionFilter
    {
        private ILogger _logger;

        /// <inheritdoc />
        public override void OnActionExecuting(ActionExecutingContext context)
        {
            _logger = context.HttpContext.RequestServices.GetLogger<ClassFilter>();
            _logger.LogInformation("ClassFilter - OnActionExecuting");
        }

        /// <inheritdoc />
        public override void OnActionExecuted(ActionExecutedContext context)
        {
            _logger.LogInformation("ClassFilter - OnActionExecuted");
        }

        /// <inheritdoc />
        public override void OnResultExecuting(ResultExecutingContext context)
        {
            _logger.LogInformation("ClassFilter - OnResultExecuting");
        }

        /// <inheritdoc />
        public override void OnResultExecuted(ResultExecutedContext context)
        {
            _logger.LogInformation("ClassFilter - OnResultExecuted");
        }

        /// <summary>
        /// Called after an action has thrown an <see cref="T:System.Exception" />.
        /// </summary>
        /// <param name="context">The <see cref="T:Microsoft.AspNetCore.Mvc.Filters.ExceptionContext" />.</param>
        public void OnException(ExceptionContext context)
        {
            var ex = context.Exception.Message;
            _logger.LogInformation("ClassFilter - OnException");
        }
    }

    public class MethodFilter : ActionFilterAttribute
    {
        private ILogger _logger;

        /// <inheritdoc />
        public override void OnActionExecuting(ActionExecutingContext context)
        {
            _logger = context.HttpContext.RequestServices.GetLogger<MethodFilter>();
            _logger.LogInformation("MethodFilter - OnActionExecuting");
        }

        /// <inheritdoc />
        public override void OnActionExecuted(ActionExecutedContext context)
        {
            _logger.LogInformation("MethodFilter - OnActionExecuted");
        }

        /// <inheritdoc />
        public override void OnResultExecuting(ResultExecutingContext context)
        {
            _logger.LogInformation("MethodFilter - OnResultExecuting");
        }

        /// <inheritdoc />
        public override void OnResultExecuted(ResultExecutedContext context)
        {
            _logger.LogInformation("MethodFilter - OnResultExecuted");
        }

    }

}
